import 'dart:async';

import 'package:_private_chat/constant.dart';
import 'package:_private_chat/options.dart';
import 'package:_private_core/_private_core.dart';
import 'package:_private_core/widgets/widgets.dart';
import 'package:flutter/material.dart';
import 'package:gap/gap.dart';
import 'package:_private_chat/chat_detail/message.dart';
import 'package:_private_chat/chat_detail/widgets/widget_message_content.dart';
import 'package:_private_chat/_private_chat.dart';
import 'package:_private_chat/utils/utils.dart';

class WidgetActionsMessage extends StatefulWidget {
  final dynamic data;
  final Widget child;
  const WidgetActionsMessage(
      {Key? key, required this.child, required this.data})
      : super(key: key);

  @override
  State<WidgetActionsMessage> createState() => _WidgetActionsMessageState();
}

class _WidgetActionsMessageState extends State<WidgetActionsMessage> {
  int get timestamp => widget.data[kDbTIMESTAMP];
  bool get isMe => widget.data[kDbFROM] == loggedFirebaseId;
  BorderRadius get borderRadius => BorderRadius.circular(12);

  bool get inputLocked => chatDetailBloc.state.defaultMessage?.lockAction == 1;
  bool get isEnableCopyEdit =>
      widget.data[kDbMESSAGETYPE] == MessageType.text.index;
  bool get isImage => widget.data[kDbMESSAGETYPE] == MessageType.image.index;

  double get widthSizeAction => 8 * 3.0 + 32 * chatEmojiRectionAssets.length;

  bool _isHover = false;
  ValueNotifier isHover = ValueNotifier(false);
  Timer? timer;

  @override
  Widget build(BuildContext context) {
    return WidgetOverlayActions(
      gestureType: GestureType.rightClick,
      builder: (child, size, childPosition, pointerPosition, animationValue,
          hide, context) {
            print("debug: ${MediaQuery.sizeOf(context).width} ${pointerPosition!.dx} ${pointerPosition!.dy}")
;        if (MediaQuery.sizeOf(context).width <
            pointerPosition!.dx + widthSizeAction * 1.05) {
          return Positioned(
            top: pointerPosition.dy + 4,
            right: 0,
            child: _buildActions(),
          );
        } else {
          return Positioned(
            top: pointerPosition.dy + 4,
            left: pointerPosition.dx,
            child: _buildActions(),
          );
        }
      },
      child: widget.child,
    );
    return MouseRegion(
      onEnter: (event) {
        _isHover = true;
        timer?.cancel();
        timer = Timer(const Duration(milliseconds: 300), () {
          if (_isHover) {
            isHover.value = true;
          }
        });
      },
      onExit: (event) {
        _isHover = false;
        isHover.value = false;
      },
      child: Row(
        mainAxisSize: MainAxisSize.min,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          if (isMe) _buildActions(),
          widget.child,
          if (!isMe) _buildActions(),
        ],
      ),
    );
  }

  Widget get _space => const Gap(8);

  Widget _buildActions() {
    return ValueListenableBuilder(
      valueListenable: isHover,
      builder: (context, value, child) {
        return AnimatedSize(
          duration: const Duration(milliseconds: 200),
          child: !isHover.value ? const SizedBox() : child!,
        );
      },
      child: Padding(
        padding: const EdgeInsets.symmetric(horizontal: 8),
        child: Column(
          crossAxisAlignment:
              isMe ? CrossAxisAlignment.end : CrossAxisAlignment.start,
          children: [
            _space,
            Container(
              height: 36,
              padding: const EdgeInsets.symmetric(horizontal: 4, vertical: 4),
              decoration: BoxDecoration(
                color: appColors.appBackground,
                borderRadius: borderRadius,
                boxShadow: appBoxShadow,
              ),
              child: SingleChildScrollView(
                scrollDirection: Axis.horizontal,
                child: Row(
                    children: chatEmojiRectionAssets
                        .mapIndexed<Widget>((e, index) =>
                            WidgetInkWellTransparent(
                              onTap: () {
                                chatDetailBloc.add(
                                    ChatDetailSendReactionMessagesEvent(
                                        widget.data, index));
                              },
                              child: Padding(
                                padding:
                                    const EdgeInsets.symmetric(horizontal: 2),
                                child: WidgetAppLottie(assetlottie('emoji/$e'),
                                    package: '_private_chat',
                                    width: 28,
                                    height: 28),
                              ),
                            ))
                        .toList()),
              ),
            ),
            _space,
            Container(
              padding: const EdgeInsets.symmetric(vertical: 4, horizontal: 8),
              decoration: BoxDecoration(
                color: appColors.appBackground,
                borderRadius: borderRadius,
                boxShadow: appBoxShadow,
              ),
              child: Row(
                children: [
                  WidgetInkWellTransparent(
                    onTap: () {
                      chatDetailBloc
                          .add(ChatDetailReplyMessagesEvent(widget.data));
                    },
                    child: Padding(
                      padding: const EdgeInsets.symmetric(
                          horizontal: 4, vertical: 4),
                      child: WidgetAppSVG(
                        assetsvg('reply'),
                        package: '_private_chat',
                        width: 24,
                      ),
                    ),
                  ),
                  WidgetInkWellTransparent(
                    onTap: () {
                      chatDetailBloc
                          .add(ChatDetailEditMessagesEvent(widget.data));
                    },
                    child: Padding(
                      padding: const EdgeInsets.symmetric(
                          horizontal: 4, vertical: 4),
                      child: WidgetAppSVG(
                        assetsvg('edit'),
                        package: '_private_chat',
                        width: 24,
                      ),
                    ),
                  )
                      .opacity(isEnableCopyEdit ? 1 : .6)
                      .ignore(!isEnableCopyEdit),
                  WidgetInkWellTransparent(
                    onTap: () {
                      chatDetailBloc.add(
                          ChatDetailCopyMessageEvent(widget.data[kDbCONTENT]));
                    },
                    child: Padding(
                      padding: const EdgeInsets.symmetric(
                          horizontal: 4, vertical: 4),
                      child: WidgetAppSVG(
                        assetsvg('copy'),
                        package: '_private_chat',
                        width: 24,
                      ),
                    ),
                  )
                      .opacity(isEnableCopyEdit ? 1 : .6)
                      .ignore(!isEnableCopyEdit),
                  WidgetInkWellTransparent(
                    onTap: () {
                      chatDetailBloc.add(ChatDeleteMessagesEvent(timestamp));
                    },
                    child: Padding(
                      padding: const EdgeInsets.symmetric(
                          horizontal: 4, vertical: 4),
                      child: WidgetAppSVG(
                        assetsvg('delete'),
                        package: '_private_chat',
                        width: 24,
                      ),
                    ),
                  ),
                  if (isImage) ...[
                    Container(
                      height: 16,
                      width: .8,
                      color: appColors.text.withOpacity(0.1),
                      margin: const EdgeInsets.symmetric(horizontal: 8),
                    ),
                    WidgetInkWellTransparent(
                      onTap: () {
                        chatOptions.downloadFromUrl?.call(
                            widget.data[kDbCONTENTURL],
                            name: widget.data[kDbCONTENTFILEINFOS]
                                ?[kDbFILEFILENAME]);
                      },
                      child: Padding(
                        padding: const EdgeInsets.symmetric(
                            horizontal: 4, vertical: 4),
                        child: WidgetAppSVG(
                          assetsvg('download'),
                          package: '_private_chat',
                          width: 24,
                        ),
                      ),
                    ),
                  ]
                ],
              ),
            ),
            _space,
          ],
        ),
      ),
    );
  }
}

get appBoxShadow => [
      BoxShadow(
        color: appColors.text.withOpacity(0.06),
        blurRadius: 4,
        spreadRadius: 0,
      )
    ];
